#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include "types.h"
#include "spi_comms.h"

/* defines section        */
#define COM_NAME_POS         1
#define SERIAL_IDLE          0x00
#define ARGS_REQUIRED        3
#define USAGE                "new_fpga_flash COM1 program.bin"
#define FILE1                2
#define PAGE_SIZE            256
#define MAX_MEMORY_SIZE      2048
#define COM_NAME_MAX_LEN     6
#define COM_NAME             "COM"
#define COM_NAME_LEN         3

/* global variables */

HANDLE port_handle = NULL;

/* local variables */

/* None */

/* prototypes section */

/* None */

int main(int argc, char *argv[])
{
	DCB dcb;
	S32 status = 0;
	FILE *file_ptr = NULL;
    U8 *program_code = NULL;
    U8 buffer[PAGE_SIZE];
    long size = 0;
    S32 num_pages = 0;
    S32 page_num = 0;
    S32 i = 0;
    S32 a = 0;
    COMMTIMEOUTS timeouts;
	
	/* check we have a program to send over */
    if(argc < ARGS_REQUIRED)
    {
        printf("Error wrong number of arguments passed \n");
        printf(USAGE);
        return EXIT_FAILURE;            
    }
  
    file_ptr = fopen(argv[FILE1], "rb");
  
    if(file_ptr == NULL)
    {
        printf("Cant open %s \n",argv[FILE1]);
        return EXIT_FAILURE; 
    }
  
    /* get the size of the program */
    fseek(file_ptr, 0, SEEK_END);
    size = ftell(file_ptr) + (PAGE_SIZE - 1);
    fseek(file_ptr, 0, SEEK_SET);
    num_pages = size / PAGE_SIZE;
    
    /* check program can fit inside the serail flash chip */
    if(num_pages >= MAX_MEMORY_SIZE)
    {
        printf("\n Error program greater than 512kb program don't fit\n");
        return EXIT_FAILURE;              
    }
      
    /* get some memory to hold the program code */
    program_code = malloc(size);
    if(program_code == NULL)
    {
        fclose(file_ptr);
        printf("Cant malloc memory for program ! \n");
        return EXIT_FAILURE;             
    }
    
    /* first clear memory space used */
    memset(program_code, 0xff, size);
    
    /* store the program code in memory and close file */
    fread(program_code, (size - (PAGE_SIZE - 1)), 1, file_ptr);
    fclose(file_ptr); 
    
    /* did user pass a valid com name ? */
    if((strlen(argv[COM_NAME_POS]) > COM_NAME_MAX_LEN) || 
            (strncmp(argv[COM_NAME_POS], COM_NAME, COM_NAME_LEN) != 0))
    {
        printf("Error com port name needs to be between COM1 and COM255 \n");
        free(program_code);
        return EXIT_FAILURE; 
    }
	
    port_handle = CreateFile(argv[COM_NAME_POS],              /* Specify port                          */
                             GENERIC_READ | GENERIC_WRITE,    /* Specify mode that open device.        */
                             0,                               /* the devide isn't shared.              */
                             NULL,                            /* the object gets a default security.   */ 
                             OPEN_EXISTING,                   /* Specify which action to take on file. */ 
                             0,                               /* default.                              */
                             NULL);                           /* default.                              */
    
    if(port_handle == INVALID_HANDLE_VALUE)
    {
        printf("Couldnt open handle to com port %s\n",argv[COM_NAME_POS]);
		free(program_code); 
        return EXIT_FAILURE; 
    }
    
    /*  Initialize the DCB structure. */
    SecureZeroMemory(&dcb, sizeof(DCB));
    dcb.DCBlength = sizeof(DCB);
    
    /* Get current configuration of serial communication port. */
    if(GetCommState(port_handle, &dcb) == 0)
    {
        printf("Couldnt get config of com port %s",argv[COM_NAME_POS]);
        free(program_code);
        return EXIT_FAILURE; 
    }
    
    /*  Fill in some DCB values and set the com state:        */ 
    /*  115200 bps, 8 data bits, no parity, and 1 stop bit.   */
    dcb.BaudRate = CBR_115200;    /*  baud rate               */
    dcb.ByteSize = 8;             /*  data size, xmit and rcv */
    dcb.Parity   = NOPARITY;      /*  parity bit              */
    dcb.StopBits = ONESTOPBIT;    /*  stop bit                */
    
    if(SetCommState(port_handle,&dcb) == 0)
    {
        printf("Couldnt set config of com port %s",argv[COM_NAME_POS]);
        free(program_code);
        return EXIT_FAILURE; 
    }
    
    if(GetCommTimeouts(port_handle,&timeouts) == 0)
    {
    	printf("Couldnt get timing config of com port %s",argv[COM_NAME_POS]);
        free(program_code);
        return EXIT_FAILURE; 
	}
    
    timeouts.ReadIntervalTimeout = 50;
    timeouts.ReadTotalTimeoutConstant = 50;
    timeouts.ReadTotalTimeoutMultiplier = 10;
    timeouts.WriteTotalTimeoutConstant = 50;
    timeouts.WriteTotalTimeoutMultiplier = 10;
 
    if(SetCommTimeouts(port_handle, &timeouts) == 0) 
    {
        printf("Couldnt set timing config of com port %s",argv[COM_NAME_POS]);
        free(program_code);
        return EXIT_FAILURE; 
    }
    
    /* check chip is in idle state before executing instructions */
    status = mc_read_status();
    
    if(status != SERIAL_IDLE)
    {
        printf("serial flash not idle! reporting state 0x%02x\n\n",status);
        free(program_code);
        return EXIT_FAILURE;            
    } 
    else
    {
        printf("serial flash in idle state 0x%02x\n\n",status);
    }
	
	printf("Bulk erasing flash checking status... \n");
    mc_erase_bulk();
	
	/* wait for bulk erase to complete max time from datasheet */
	/* is 10 seconds normally its a bit less so wait 8 seconds */
	Sleep(8000);  
    
    while (1)
    {
        status = mc_read_status();
        printf("\rstatus = %02x ", status);
	
        /* check to see if serial flash has finshed erasing page */	
        if(SERIAL_IDLE == status)
        {
            break;
        }
      
        Sleep(100);
    }
    printf("\n done! \n");
    
    /* program code */
    printf("programming code..\n");
  
    printf("%d pages ...\n", num_pages);
    
    /* start programming from page 0 */
    page_num = 0;
    
    for (i = 0; i < num_pages; i++)
    {
        printf("\rprogramming at %4d", (page_num / PAGE_SIZE));
        mc_program(page_num, program_code + (i * PAGE_SIZE), PAGE_SIZE);
		    
        while (1)
	    {
            /* check to see if serial flash has finshed programming page */   
            if(mc_read_status() ==  SERIAL_IDLE)
            {
                break;
            }
        }  
				
        page_num += PAGE_SIZE ;
    }
    
    printf("\ndone, verifying...\n");
    memset(buffer, 0, PAGE_SIZE);
    
    /* check each page make sure everything is lovely */
    for (i = 0; i < num_pages; i++)
    {   	   
		mc_read((i * PAGE_SIZE), buffer, PAGE_SIZE);
		
		if(memcmp(buffer, program_code + (i * PAGE_SIZE), PAGE_SIZE) != 0)
		{
			printf("!!! page %d failed to verify !!!\n", i);

            /* find bytes in page that failed to program */
			for(a = 0; a < PAGE_SIZE; a++)
			{
				if(buffer[a] != program_code[(i * PAGE_SIZE) + a])
				{
					printf("%03x: %02x != %02x\n", a, buffer[a], program_code[(i * PAGE_SIZE) + a]);
                }
			}
			
			break;
		}
        else
		{
			printf("\rpage %d ok", i + 1);
		}
    }
    printf("\ndone!");
	
    /* job done close port. */
    if(CloseHandle(port_handle) == 0)
    {
        printf("Couldnt close com port %s",argv[COM_NAME_POS]);
        free(program_code);
        return EXIT_FAILURE; 
    }
      
    /* free memory and return */
    free(program_code);
    return EXIT_SUCCESS; 
}
